@@ -45,6 +45,8 @@
 #include "nasmlib.h"
 #include "preproc.h"
 #include "insnsi.h"		/* For enum opcode */
+#include "directives.h"		/* For enum directive */
+#include "opflags.h"
 
 #define NO_SEG -1L              /* null segment value */
 #define SEG_ABS 0x40000000L     /* mask for far-absolute segments */
@@ -121,9 +123,10 @@ typedef bool (*lfunc) (char *label, int32_t *segment, int64_t *offset);
  * should affect the local-label system), or something odder like
  * an EQU or a segment-base symbol, which shouldn't.
  */
-typedef void (*ldfunc) (char *label, int32_t segment, int64_t offset,
-                        char *special, bool is_norm, bool isextrn,
-                        struct ofmt * ofmt, efunc error);
+typedef void (*ldfunc)(char *label, int32_t segment, int64_t offset,
+		       char *special, bool is_norm, bool isextrn);
+void define_label(char *label, int32_t segment, int64_t offset,
+		  char *special, bool is_norm, bool isextrn);
 
 /*
  * List-file generators should look like this:
@@ -353,7 +356,7 @@ typedef struct preproc_ops {
      * of the pass, an error reporting function, an evaluator
      * function, and a listing generator to talk to.
      */
-    void (*reset) (char *, int, efunc, evalfunc, ListGen *, StrList **);
+    void (*reset) (char *, int, ListGen *, StrList **);
 
     /*
      * Called to fetch a line of preprocessed source. The line
@@ -414,200 +417,6 @@ enum {
  * -----------------------------------------------------------
  */
 
-/*
- * Here we define the operand types. These are implemented as bit
- * masks, since some are subsets of others; e.g. AX in a MOV
- * instruction is a special operand type, whereas AX in other
- * contexts is just another 16-bit register. (Also, consider CL in
- * shift instructions, DX in OUT, etc.)
- *
- * The basic concept here is that
- *    (class & ~operand) == 0
- *
- * if and only if "operand" belongs to class type "class".
- *
- * The bits are assigned as follows:
- *
- * Bits 0-7, 23, 29: sizes
- *  0:  8 bits (BYTE)
- *  1: 16 bits (WORD)
- *  2: 32 bits (DWORD)
- *  3: 64 bits (QWORD)
- *  4: 80 bits (TWORD)
- *  5: FAR
- *  6: NEAR
- *  7: SHORT
- * 23: 256 bits (YWORD)
- * 29: 128 bits (OWORD)
- *
- * Bits 8-11 modifiers
- *  8: TO
- *  9: COLON
- * 10: STRICT
- * 11: (reserved)
- *
- * Bits 12-15: type of operand
- * 12: REGISTER
- * 13: IMMEDIATE
- * 14: MEMORY (always has REGMEM attribute as well)
- * 15: REGMEM (valid EA operand)
- *
- * Bits 16-19, 28: subclasses
- * With REG_CDT:
- * 16: REG_CREG (CRx)
- * 17: REG_DREG (DRx)
- * 18: REG_TREG (TRx)
-
- * With REG_GPR:
- * 16: REG_ACCUM  (AL, AX, EAX, RAX)
- * 17: REG_COUNT  (CL, CX, ECX, RCX)
- * 18: REG_DATA   (DL, DX, EDX, RDX)
- * 19: REG_HIGH   (AH, CH, DH, BH)
- * 28: REG_NOTACC (not REG_ACCUM)
- *
- * With REG_SREG:
- * 16: REG_CS
- * 17: REG_DESS (DS, ES, SS)
- * 18: REG_FSGS
- * 19: REG_SEG67
- *
- * With FPUREG:
- * 16: FPU0
- *
- * With XMMREG:
- * 16: XMM0
- *
- * With YMMREG:
- * 16: YMM0
- *
- * With MEMORY:
- * 16: MEM_OFFS (this is a simple offset)
- * 17: IP_REL (IP-relative offset)
- *
- * With IMMEDIATE:
- * 16: UNITY (1)
- * 17: BYTENESS16 (-128..127)
- * 18: BYTENESS32 (-128..127)
- * 19: BYTENESS64 (-128..127)
- *
- * Bits 20-22, 24-27: register classes
- * 20: REG_CDT (CRx, DRx, TRx)
- * 21: RM_GPR (REG_GPR) (integer register)
- * 22: REG_SREG
- * 24: FPUREG
- * 25: RM_MMX (MMXREG)
- * 26: RM_XMM (XMMREG)
- * 27: RM_YMM (YMMREG)
- *
- * Bit 31 is currently unallocated.
- *
- * 30: SAME_AS
- * Special flag only used in instruction patterns; means this operand
- * has to be identical to another operand.  Currently only supported
- * for registers.
- */
-
-typedef uint32_t opflags_t;
-
-/* Size, and other attributes, of the operand */
-#define BITS8		0x00000001U
-#define BITS16		0x00000002U
-#define BITS32		0x00000004U
-#define BITS64		0x00000008U   /* x64 and FPU only */
-#define BITS80		0x00000010U   /* FPU only */
-#define BITS128		0x20000000U
-#define BITS256		0x00800000U
-#define FAR		0x00000020U   /* grotty: this means 16:16 or */
-                                       /* 16:32, like in CALL/JMP */
-#define NEAR		0x00000040U
-#define SHORT		0x00000080U   /* and this means what it says :) */
-
-#define SIZE_MASK	0x208000FFU   /* all the size attributes */
-
-/* Modifiers */
-#define MODIFIER_MASK	0x00000f00U
-#define TO		0x00000100U   /* reverse effect in FADD, FSUB &c */
-#define COLON		0x00000200U   /* operand is followed by a colon */
-#define STRICT		0x00000400U   /* do not optimize this operand */
-
-/* Type of operand: memory reference, register, etc. */
-#define OPTYPE_MASK	0x0000f000U
-#define REGISTER	0x00001000U   /* register number in 'basereg' */
-#define IMMEDIATE	0x00002000U
-#define MEMORY		0x0000c000U
-#define REGMEM		0x00008000U   /* for r/m, ie EA, operands */
-
-/* Register classes */
-#define REG_EA		0x00009000U   /* 'normal' reg, qualifies as EA */
-#define RM_GPR		0x00208000U   /* integer operand */
-#define REG_GPR		0x00209000U   /* integer register */
-#define REG8		0x00209001U   /*  8-bit GPR  */
-#define REG16		0x00209002U   /* 16-bit GPR */
-#define REG32		0x00209004U   /* 32-bit GPR */
-#define REG64		0x00209008U   /* 64-bit GPR */
-#define FPUREG		0x01001000U   /* floating point stack registers */
-#define FPU0		0x01011000U   /* FPU stack register zero */
-#define RM_MMX		0x02008000U   /* MMX operand */
-#define MMXREG		0x02009000U   /* MMX register */
-#define RM_XMM		0x04008000U   /* XMM (SSE) operand */
-#define XMMREG		0x04009000U   /* XMM (SSE) register */
-#define XMM0		0x04019000U   /* XMM register zero */
-#define RM_YMM		0x08008000U   /* YMM (AVX) operand */
-#define YMMREG		0x08009000U   /* YMM (AVX) register */
-#define YMM0		0x08019000U   /* YMM register zero */
-#define REG_CDT		0x00101004U   /* CRn, DRn and TRn */
-#define REG_CREG	0x00111004U   /* CRn */
-#define REG_DREG	0x00121004U   /* DRn */
-#define REG_TREG	0x00141004U   /* TRn */
-#define REG_SREG	0x00401002U   /* any segment register */
-#define REG_CS		0x00411002U   /* CS */
-#define REG_DESS	0x00421002U   /* DS, ES, SS */
-#define REG_FSGS	0x00441002U   /* FS, GS */
-#define REG_SEG67	0x00481002U   /* Unimplemented segment registers */
-
-#define REG_RIP		0x00801008U   /* RIP relative addressing */
-#define REG_EIP		0x00801004U   /* EIP relative addressing */
-
-/* Special GPRs */
-#define REG_SMASK	0x100f0000U   /* a mask for the following */
-#define REG_ACCUM	0x00219000U   /* accumulator: AL, AX, EAX, RAX */
-#define REG_AL		0x00219001U
-#define REG_AX		0x00219002U
-#define REG_EAX		0x00219004U
-#define REG_RAX		0x00219008U
-#define REG_COUNT	0x10229000U   /* counter: CL, CX, ECX, RCX */
-#define REG_CL		0x10229001U
-#define REG_CX		0x10229002U
-#define REG_ECX		0x10229004U
-#define REG_RCX		0x10229008U
-#define REG_DL		0x10249001U   /* data: DL, DX, EDX, RDX */
-#define REG_DX		0x10249002U
-#define REG_EDX		0x10249004U
-#define REG_RDX		0x10249008U
-#define REG_HIGH	0x10289001U   /* high regs: AH, CH, DH, BH */
-#define REG_NOTACC	0x10000000U   /* non-accumulator register */
-#define REG8NA		0x10209001U   /*  8-bit non-acc GPR  */
-#define REG16NA		0x10209002U   /* 16-bit non-acc GPR */
-#define REG32NA		0x10209004U   /* 32-bit non-acc GPR */
-#define REG64NA		0x10209008U   /* 64-bit non-acc GPR */
-
-/* special types of EAs */
-#define MEM_OFFS	0x0001c000U   /* simple [address] offset - absolute! */
-#define IP_REL		0x0002c000U   /* IP-relative offset */
-
-/* memory which matches any type of r/m operand */
-#define MEMORY_ANY	(MEMORY|RM_GPR|RM_MMX|RM_XMM|RM_YMM)
-
-/* special type of immediate operand */
-#define UNITY		0x00012000U   /* for shift/rotate instructions */
-#define SBYTE16		0x00022000U   /* for op r16,immediate instrs. */
-#define SBYTE32		0x00042000U   /* for op r32,immediate instrs. */
-#define SBYTE64		0x00082000U   /* for op r64,immediate instrs. */
-#define BYTENESS	0x000e0000U   /* for testing for byteness */
-
-/* special flags */
-#define SAME_AS		0x40000000U
-
 /* Register names automatically generated from regs.dat */
 #include "regs.h"
 
@@ -680,7 +489,7 @@ enum eval_hint {                /* values for `hinttype' */
 };
 
 typedef struct operand {	/* operand to an instruction */
-    int32_t type;               /* type of operand */
+    opflags_t type;             /* type of operand */
     int disp_size;              /* 0 means default; 16; 32; 64 */
     enum reg_enum basereg, indexreg; /* address registers */
     int scale;			/* index scale */
@@ -785,7 +594,7 @@ struct ofmt {
      * an output format, be sure to set this to whatever default you want
      *
      */
-    struct dfmt *current_dfmt;
+    const struct dfmt *current_dfmt;
 
     /*
      * This, if non-NULL, is a NULL-terminated list of `char *'s
@@ -797,13 +606,10 @@ struct ofmt {
     macros_t *stdmac;
 
     /*
-     * This procedure is called at the start of an output session.
-     * It tells the output format what file it will be writing to,
-     * what routine to report errors through, and how to interface
-     * to the label manager and expression evaluator if necessary.
-     * It also gives it a chance to do other initialisation.
+     * This procedure is called at the start of an output session to set
+     * up internal parameters.
      */
-    void (*init) (FILE * fp, efunc error, ldfunc ldef, evalfunc eval);
+    void (*init)(void);
 
     /*
      * This procedure is called to pass generic information to the
@@ -904,7 +710,7 @@ struct ofmt {
      * should also return non-zero if it correctly processes the
      * directive.
      */
-    int (*directive) (char *directive, char *value, int pass);
+    int (*directive)(enum directives directive, char *value, int pass);
 
     /*
      * This procedure is called before anything else - even before
@@ -923,7 +729,7 @@ struct ofmt {
      * The parameter `outname' points to an area of storage
      * guaranteed to be at least FILENAME_MAX in size.
      */
-    void (*filename) (char *inname, char *outname, efunc error);
+    void (*filename) (char *inname, char *outname);
 
     /*
      * This procedure is called after assembly finishes, to allow
@@ -937,6 +743,8 @@ struct ofmt {
     void (*cleanup) (int debuginfo);
 };
 
+extern struct ofmt *ofmt;
+extern FILE *ofile;
 
 /*
  * ------------------------------------------------------------
@@ -958,17 +766,15 @@ struct dfmt {
     const char *shortname;
 
     /*
-     * init - called initially to set up local pointer to object format,
-     * void pointer to implementation defined data, file pointer (which
-     * probably won't be used, but who knows?), and error function.
+     * init - called initially to set up local pointer to object format.
      */
-    void (*init) (struct ofmt * of, void *id, FILE * fp, efunc error);
+    void (*init)(void);
 
     /*
      * linenum - called any time there is output with a change of
      * line number or file.
      */
-    void (*linenum) (const char *filename, int32_t linenumber, int32_t segto);
+    void (*linenum)(const char *filename, int32_t linenumber, int32_t segto);
 
     /*
      * debug_deflabel - called whenever a label is defined. Parameters
@@ -976,8 +782,8 @@ struct dfmt {
      * would be called before the output format version.
      */
 
-    void (*debug_deflabel) (char *name, int32_t segment, int64_t offset,
-                            int is_global, char *special);
+    void (*debug_deflabel)(char *name, int32_t segment, int64_t offset,
+			   int is_global, char *special);
     /*
      * debug_directive - called whenever a DEBUG directive other than 'LINE'
      * is encountered. 'directive' contains the first parameter to the
@@ -986,27 +792,29 @@ struct dfmt {
      * function with 'directive' equal to "VAR" and 'params' equal to
      * "_somevar:int".
      */
-    void (*debug_directive) (const char *directive, const char *params);
+    void (*debug_directive)(const char *directive, const char *params);
 
     /*
      * typevalue - called whenever the assembler wishes to register a type
      * for the last defined label.  This routine MUST detect if a type was
      * already registered and not re-register it.
      */
-    void (*debug_typevalue) (int32_t type);
+    void (*debug_typevalue)(int32_t type);
 
     /*
      * debug_output - called whenever output is required
      * 'type' is the type of info required, and this is format-specific
      */
-    void (*debug_output) (int type, void *param);
+    void (*debug_output)(int type, void *param);
 
     /*
      * cleanup - called after processing of file is complete
      */
-    void (*cleanup) (void);
-
+    void (*cleanup)(void);
 };
+
+extern const struct dfmt *dfmt;
+
 /*
  * The type definition macros
  * for debugging
