@@ -51,6 +51,7 @@
 #include "nasmlib.h"
 #include "saa.h"
 #include "raa.h"
+#include "eval.h"
 #include "output/outform.h"
 #include "output/outlib.h"
 
@@ -207,10 +208,6 @@ static struct RAA *extsyms;
 static struct SAA *strs;
 static uint32_t strslen;
 
-static FILE *machofp;
-static efunc error;
-static evalfunc evaluate;
-
 extern struct ofmt of_macho;
 
 /* Global file information. This should be cleaned up into either
@@ -228,11 +225,8 @@ uint32_t rel_padcnt = 0;
     strncpy(xdst, xsrc, sizeof(xdst));	/* copy over string */		\
     xdst[sizeof(xdst) - 1] = '\0';      /* proper null-termination */
 
-#define align(x, y)							\
-    (((x) + (y) - 1) & ~((y) - 1))      /* align x to multiple of y */
-
 #define alignint32_t(x)							\
-    align(x, sizeof(int32_t))      /* align x to int32_t boundary */
+    ALIGN(x, sizeof(int32_t))      /* align x to int32_t boundary */
 
 static void debug_reloc (struct reloc *);
 static void debug_section_relocs (struct section *) _unused;
@@ -322,23 +316,16 @@ static uint8_t get_section_fileindex_by_index(const int32_t index)
             return i;
 
     if (i == MAX_SECT)
-        error(ERR_WARNING,
+        nasm_error(ERR_WARNING,
               "too many sections (>255) - clipped by fileindex");
 
     return NO_SECT;
 }
 
-static void macho_init(FILE * fp, efunc errfunc, ldfunc ldef,
-                       evalfunc eval)
+static void macho_init(void)
 {
     char zero = 0;
 
-    machofp = fp;
-    error = errfunc;
-    evaluate = eval;
-
-    (void)ldef;                 /* placate optimisers */
-
     sects = NULL;
     sectstail = &sects;
 
@@ -357,13 +344,6 @@ static void macho_init(FILE * fp, efunc errfunc, ldfunc ldef,
     strslen = 1;
 }
 
-static int macho_setinfo(enum geninfo type, char **val)
-{
-    (void)type;
-    (void)val;
-    return 0;
-}
-
 static void sect_write(struct section *sect,
                        const uint8_t *data, uint32_t len)
 {
@@ -428,13 +408,13 @@ static void macho_output(int32_t secto, const void *data,
 
     if (wrt != NO_SEG) {
         wrt = NO_SEG;
-        error(ERR_NONFATAL, "WRT not supported by Mach-O output format");
+        nasm_error(ERR_NONFATAL, "WRT not supported by Mach-O output format");
         /* continue to do _something_ */
     }
 
     if (secto == NO_SEG) {
         if (type != OUT_RESERVE)
-            error(ERR_NONFATAL, "attempt to assemble code in "
+            nasm_error(ERR_NONFATAL, "attempt to assemble code in "
                   "[ABSOLUTE] space");
 
         return;
@@ -443,19 +423,19 @@ static void macho_output(int32_t secto, const void *data,
     s = get_section_by_index(secto);
 
     if (s == NULL) {
-        error(ERR_WARNING, "attempt to assemble code in"
+        nasm_error(ERR_WARNING, "attempt to assemble code in"
               " section %d: defaulting to `.text'", secto);
         s = get_section_by_name("__TEXT", "__text");
 
         /* should never happen */
         if (s == NULL)
-            error(ERR_PANIC, "text section not found");
+            nasm_error(ERR_PANIC, "text section not found");
     }
 
     sbss = get_section_by_name("__DATA", "__bss");
 
     if (s == sbss && type != OUT_RESERVE) {
-        error(ERR_WARNING, "attempt to initialize memory in the"
+        nasm_error(ERR_WARNING, "attempt to initialize memory in the"
               " BSS section: ignored");
         s->size += realsize(type, size);
         return;
@@ -464,7 +444,7 @@ static void macho_output(int32_t secto, const void *data,
     switch (type) {
     case OUT_RESERVE:
         if (s != sbss) {
-            error(ERR_WARNING, "uninitialized space declared in"
+            nasm_error(ERR_WARNING, "uninitialized space declared in"
                   " %s section: zeroing",
                   get_section_name_by_index(secto));
 
@@ -476,7 +456,7 @@ static void macho_output(int32_t secto, const void *data,
 
     case OUT_RAWDATA:
         if (section != NO_SEG)
-            error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
+            nasm_error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
 
         sect_write(s, data, size);
         break;
@@ -486,7 +466,7 @@ static void macho_output(int32_t secto, const void *data,
 
         if (section != NO_SEG) {
             if (section % 2) {
-                error(ERR_NONFATAL, "Mach-O format does not support"
+                nasm_error(ERR_NONFATAL, "Mach-O format does not support"
                       " section base references");
             } else
                 add_reloc(s, section, 0, size);
@@ -499,10 +479,10 @@ static void macho_output(int32_t secto, const void *data,
 
     case OUT_REL2ADR:
         if (section == secto)
-            error(ERR_PANIC, "intra-section OUT_REL2ADR");
+            nasm_error(ERR_PANIC, "intra-section OUT_REL2ADR");
 
         if (section != NO_SEG && section % 2) {
-            error(ERR_NONFATAL, "Mach-O format does not support"
+            nasm_error(ERR_NONFATAL, "Mach-O format does not support"
                   " section base references");
         } else
             add_reloc(s, section, 1, 2);
@@ -514,10 +494,10 @@ static void macho_output(int32_t secto, const void *data,
 
     case OUT_REL4ADR:
         if (section == secto)
-            error(ERR_PANIC, "intra-section OUT_REL4ADR");
+            nasm_error(ERR_PANIC, "intra-section OUT_REL4ADR");
 
         if (section != NO_SEG && section % 2) {
-            error(ERR_NONFATAL, "Mach-O format does not support"
+            nasm_error(ERR_NONFATAL, "Mach-O format does not support"
                   " section base references");
         } else
             add_reloc(s, section, 1, 4);
@@ -528,7 +508,7 @@ static void macho_output(int32_t secto, const void *data,
         break;
 
     default:
-        error(ERR_PANIC, "unknown output type?");
+        nasm_error(ERR_PANIC, "unknown output type?");
         break;
     }
 }
@@ -594,14 +574,14 @@ static int32_t macho_section(char *name, int pass, int *bits)
                         newAlignment = exact_log2(value);
 
                         if (0 != *end) {
-                            error(ERR_PANIC,
+                            nasm_error(ERR_PANIC,
                                   "unknown or missing alignment value \"%s\" "
                                       "specified for section \"%s\"",
                                   currentAttribute + 6,
                                   name);
                             return NO_SEG;
                         } else if (0 > newAlignment) {
-                            error(ERR_PANIC,
+                            nasm_error(ERR_PANIC,
                                   "alignment of %d (for section \"%s\") is not "
                                       "a power of two",
                                   value,
@@ -612,7 +592,7 @@ static int32_t macho_section(char *name, int pass, int *bits)
                         if ((-1 != originalIndex)
                             && (s->align != newAlignment)
                            && (s->align != -1)) {
-                            error(ERR_PANIC,
+                            nasm_error(ERR_PANIC,
                                   "section \"%s\" has already been specified "
                                       "with alignment %d, conflicts with new "
                                       "alignment of %d",
@@ -626,7 +606,7 @@ static int32_t macho_section(char *name, int pass, int *bits)
                     } else if (!nasm_stricmp("data", currentAttribute)) {
                         /* Do nothing; 'data' is implicit */
                     } else {
-                        error(ERR_PANIC,
+                        nasm_error(ERR_PANIC,
                               "unknown section attribute %s for section %s",
                               currentAttribute,
                               name);
@@ -639,7 +619,7 @@ static int32_t macho_section(char *name, int pass, int *bits)
         }
     }
 
-    error(ERR_PANIC, "invalid section name %s", name);
+    nasm_error(ERR_PANIC, "invalid section name %s", name);
     return NO_SEG;
 }
 
@@ -649,13 +629,13 @@ static void macho_symdef(char *name, int32_t section, int64_t offset,
     struct symbol *sym;
 
     if (special) {
-        error(ERR_NONFATAL, "The Mach-O output format does "
+        nasm_error(ERR_NONFATAL, "The Mach-O output format does "
               "not support any special symbol types");
         return;
     }
 
     if (is_global == 3) {
-        error(ERR_NONFATAL, "The Mach-O format does not "
+        nasm_error(ERR_NONFATAL, "The Mach-O format does not "
               "(yet) support forward reference fixups.");
         return;
     }
@@ -706,7 +686,7 @@ static void macho_symdef(char *name, int32_t section, int64_t offset,
                 /* give an error on unfound section if it's not an
                  ** external or common symbol (assemble_file() does a
                  ** seg_alloc() on every call for them) */
-                error(ERR_PANIC, "in-file index for section %d not found",
+                nasm_error(ERR_PANIC, "in-file index for section %d not found",
                       section);
             }
         }
@@ -720,17 +700,9 @@ static int32_t macho_segbase(int32_t section)
     return section;
 }
 
-static int macho_directive(char *directive, char *value, int pass)
+static void macho_filename(char *inname, char *outname)
 {
-    (void)directive;
-    (void)value;
-    (void)pass;
-    return 0;
-}
-
-static void macho_filename(char *inname, char *outname, efunc error)
-{
-    standard_extension(inname, outname, ".o", error);
+    standard_extension(inname, outname, ".o");
 }
 
 extern macros_t macho_stdmac[];
@@ -857,7 +829,7 @@ static void macho_calculate_sizes (void)
         if (s->align == -1)
             s->align = DEFAULT_SECTION_ALIGNMENT;
         if(s->align) {
-            uint32_t newaddr = align(s->addr, 1 << s->align);
+            uint32_t newaddr = ALIGN(s->addr, 1 << s->align);
             pad = newaddr - s->addr;
             s->addr = newaddr;
         }
@@ -884,13 +856,13 @@ static void macho_calculate_sizes (void)
 
 static void macho_write_header (void)
 {
-    fwriteint32_t(MH_MAGIC, machofp);	/* magic */
-    fwriteint32_t(CPU_TYPE_I386, machofp);	/* CPU type */
-    fwriteint32_t(CPU_SUBTYPE_I386_ALL, machofp);	/* CPU subtype */
-    fwriteint32_t(MH_OBJECT, machofp);	/* Mach-O file type */
-    fwriteint32_t(head_ncmds, machofp);	/* number of load commands */
-    fwriteint32_t(head_sizeofcmds, machofp);	/* size of load commands */
-    fwriteint32_t(0, machofp);	/* no flags */
+    fwriteint32_t(MH_MAGIC, ofile);	/* magic */
+    fwriteint32_t(CPU_TYPE_I386, ofile);	/* CPU type */
+    fwriteint32_t(CPU_SUBTYPE_I386_ALL, ofile);	/* CPU subtype */
+    fwriteint32_t(MH_OBJECT, ofile);	/* Mach-O file type */
+    fwriteint32_t(head_ncmds, ofile);	/* number of load commands */
+    fwriteint32_t(head_sizeofcmds, ofile);	/* size of load commands */
+    fwriteint32_t(0, ofile);	/* no flags */
 }
 
 /* Write out the segment load command at offset.  */
@@ -901,56 +873,56 @@ static uint32_t macho_write_segment (uint32_t offset)
     uint32_t s_reloff = 0;
     struct section *s;
 
-    fwriteint32_t(LC_SEGMENT, machofp);        /* cmd == LC_SEGMENT */
+    fwriteint32_t(LC_SEGMENT, ofile);        /* cmd == LC_SEGMENT */
 
     /* size of load command including section load commands */
     fwriteint32_t(MACHO_SEGCMD_SIZE + seg_nsects *
-	       MACHO_SECTCMD_SIZE, machofp);
+	       MACHO_SECTCMD_SIZE, ofile);
 
     /* in an MH_OBJECT file all sections are in one unnamed (name
     ** all zeros) segment */
-    fwritezero(16, machofp);
-    fwriteint32_t(0, machofp); /* in-memory offset */
-    fwriteint32_t(seg_vmsize, machofp);        /* in-memory size */
-    fwriteint32_t(offset, machofp);    /* in-file offset to data */
-    fwriteint32_t(seg_filesize, machofp);      /* in-file size */
-    fwriteint32_t(VM_PROT_DEFAULT, machofp);   /* maximum vm protection */
-    fwriteint32_t(VM_PROT_DEFAULT, machofp);   /* initial vm protection */
-    fwriteint32_t(seg_nsects, machofp);        /* number of sections */
-    fwriteint32_t(0, machofp); /* no flags */
+    fwritezero(16, ofile);
+    fwriteint32_t(0, ofile); /* in-memory offset */
+    fwriteint32_t(seg_vmsize, ofile);        /* in-memory size */
+    fwriteint32_t(offset, ofile);    /* in-file offset to data */
+    fwriteint32_t(seg_filesize, ofile);      /* in-file size */
+    fwriteint32_t(VM_PROT_DEFAULT, ofile);   /* maximum vm protection */
+    fwriteint32_t(VM_PROT_DEFAULT, ofile);   /* initial vm protection */
+    fwriteint32_t(seg_nsects, ofile);        /* number of sections */
+    fwriteint32_t(0, ofile); /* no flags */
 
     /* emit section headers */
     for (s = sects; s != NULL; s = s->next) {
-	fwrite(s->sectname, sizeof(s->sectname), 1, machofp);
-	fwrite(s->segname, sizeof(s->segname), 1, machofp);
-	fwriteint32_t(s->addr, machofp);
-	fwriteint32_t(s->size, machofp);
+	fwrite(s->sectname, sizeof(s->sectname), 1, ofile);
+	fwrite(s->segname, sizeof(s->segname), 1, ofile);
+	fwriteint32_t(s->addr, ofile);
+	fwriteint32_t(s->size, ofile);
 
 	/* dummy data for zerofill sections or proper values */
 	if ((s->flags & SECTION_TYPE) != S_ZEROFILL) {
-	    fwriteint32_t(offset, machofp);
+	    fwriteint32_t(offset, ofile);
 	    /* Write out section alignment, as a power of two.
 	       e.g. 32-bit word alignment would be 2 (2^^2 = 4).  */
 	    if (s->align == -1)
 		s->align = DEFAULT_SECTION_ALIGNMENT;
-	    fwriteint32_t(s->align, machofp);
+	    fwriteint32_t(s->align, ofile);
 	    /* To be compatible with cctools as we emit
 	       a zero reloff if we have no relocations.  */
-	    fwriteint32_t(s->nreloc ? rel_base + s_reloff : 0, machofp);
-	    fwriteint32_t(s->nreloc, machofp);
+	    fwriteint32_t(s->nreloc ? rel_base + s_reloff : 0, ofile);
+	    fwriteint32_t(s->nreloc, ofile);
 
 	    offset += s->size;
 	    s_reloff += s->nreloc * MACHO_RELINFO_SIZE;
 	} else {
-	    fwriteint32_t(0, machofp);
-	    fwriteint32_t(0, machofp);
-	    fwriteint32_t(0, machofp);
-	    fwriteint32_t(0, machofp);
+	    fwriteint32_t(0, ofile);
+	    fwriteint32_t(0, ofile);
+	    fwriteint32_t(0, ofile);
+	    fwriteint32_t(0, ofile);
 	}
 
-	fwriteint32_t(s->flags, machofp);      /* flags */
-	fwriteint32_t(0, machofp);     /* reserved */
-	fwriteint32_t(0, machofp);     /* reserved */
+	fwriteint32_t(s->flags, ofile);      /* flags */
+	fwriteint32_t(0, ofile);     /* reserved */
+	fwriteint32_t(0, ofile);     /* reserved */
     }
 
     rel_padcnt = rel_base - offset;
@@ -967,14 +939,14 @@ static void macho_write_relocs (struct reloc *r)
     while (r) {
 	uint32_t word2;
 
-	fwriteint32_t(r->addr, machofp); /* reloc offset */
+	fwriteint32_t(r->addr, ofile); /* reloc offset */
 
 	word2 = r->snum;
 	word2 |= r->pcrel << 24;
 	word2 |= r->length << 25;
 	word2 |= r->ext << 27;
 	word2 |= r->type << 28;
-	fwriteint32_t(word2, machofp); /* reloc data */
+	fwriteint32_t(word2, ofile); /* reloc data */
 
 	r = r->next;
     }
@@ -1041,11 +1013,11 @@ static void macho_write_section (void)
 	}
 
 	/* dump the section data to file */
-	saa_fpwrite(s->data, machofp);
+	saa_fpwrite(s->data, ofile);
     }
 
     /* pad last section up to reloc entries on int32_t boundary */
-    fwritezero(rel_padcnt, machofp);
+    fwritezero(rel_padcnt, ofile);
 
     /* emit relocation entries */
     for (s = sects; s != NULL; s = s->next)
@@ -1065,29 +1037,32 @@ static void macho_write_symtab (void)
 
     for (sym = syms; sym != NULL; sym = sym->next) {
 	if ((sym->type & N_EXT) == 0) {
-	    fwriteint32_t(sym->strx, machofp);		/* string table entry number */
-	    fwrite(&sym->type, 1, 1, machofp);	/* symbol type */
-	    fwrite(&sym->sect, 1, 1, machofp);	/* section */
-	    fwriteint16_t(sym->desc, machofp);	/* description */
+	    fwriteint32_t(sym->strx, ofile);		/* string table entry number */
+	    fwrite(&sym->type, 1, 1, ofile);	/* symbol type */
+	    fwrite(&sym->sect, 1, 1, ofile);	/* section */
+	    fwriteint16_t(sym->desc, ofile);	/* description */
 
 	    /* Fix up the symbol value now that we know the final section
 	       sizes.  */
 	    if (((sym->type & N_TYPE) == N_SECT) && (sym->sect != NO_SECT)) {
-		for (s = sects, fi = 1;
-		     s != NULL && fi < sym->sect; s = s->next, ++fi)
-		    sym->value += s->size;
+		    for (s = sects, fi = 1; s != NULL; s = s->next, fi++) {
+		        if (fi == sym->sect) {
+		            sym->value += s->addr;
+		            break;
+		        }
+		    }
 	    }
 
-	    fwriteint32_t(sym->value, machofp);	/* value (i.e. offset) */
+	    fwriteint32_t(sym->value, ofile);	/* value (i.e. offset) */
 	}
     }
 
     for (i = 0; i < nextdefsym; i++) {
 	sym = extdefsyms[i];
-	fwriteint32_t(sym->strx, machofp);
-	fwrite(&sym->type, 1, 1, machofp);	/* symbol type */
-	fwrite(&sym->sect, 1, 1, machofp);	/* section */
-	fwriteint16_t(sym->desc, machofp);	/* description */
+	fwriteint32_t(sym->strx, ofile);
+	fwrite(&sym->type, 1, 1, ofile);	/* symbol type */
+	fwrite(&sym->sect, 1, 1, ofile);	/* section */
+	fwriteint16_t(sym->desc, ofile);	/* description */
 
 	/* Fix up the symbol value now that we know the final section
 	   sizes.  */
@@ -1097,15 +1072,15 @@ static void macho_write_symtab (void)
 		sym->value += s->size;
 	}
 
-	fwriteint32_t(sym->value, machofp);	/* value (i.e. offset) */
+	fwriteint32_t(sym->value, ofile);	/* value (i.e. offset) */
     }
 
      for (i = 0; i < nundefsym; i++) {
 	 sym = undefsyms[i];
-	 fwriteint32_t(sym->strx, machofp);
-	 fwrite(&sym->type, 1, 1, machofp);	/* symbol type */
-	 fwrite(&sym->sect, 1, 1, machofp);	/* section */
-	 fwriteint16_t(sym->desc, machofp);	/* description */
+	 fwriteint32_t(sym->strx, ofile);
+	 fwrite(&sym->type, 1, 1, ofile);	/* symbol type */
+	 fwrite(&sym->sect, 1, 1, ofile);	/* section */
+	 fwriteint16_t(sym->desc, ofile);	/* description */
 
 	 /* Fix up the symbol value now that we know the final section
 	    sizes.  */
@@ -1115,7 +1090,7 @@ static void macho_write_symtab (void)
 		 sym->value += s->size;
 	 }
 
-	 fwriteint32_t(sym->value, machofp);	/* value (i.e. offset) */
+	 fwriteint32_t(sym->value, ofile);	/* value (i.e. offset) */
      }
 }
 
@@ -1228,19 +1203,19 @@ static void macho_write (void)
     if (seg_nsects > 0)
 	offset = macho_write_segment (offset);
     else
-        error(ERR_WARNING, "no sections?");
+        nasm_error(ERR_WARNING, "no sections?");
 
     if (nsyms > 0) {
         /* write out symbol command */
-        fwriteint32_t(LC_SYMTAB, machofp); /* cmd == LC_SYMTAB */
-        fwriteint32_t(MACHO_SYMCMD_SIZE, machofp); /* size of load command */
-        fwriteint32_t(offset, machofp);    /* symbol table offset */
-        fwriteint32_t(nsyms, machofp);     /* number of symbol
+        fwriteint32_t(LC_SYMTAB, ofile); /* cmd == LC_SYMTAB */
+        fwriteint32_t(MACHO_SYMCMD_SIZE, ofile); /* size of load command */
+        fwriteint32_t(offset, ofile);    /* symbol table offset */
+        fwriteint32_t(nsyms, ofile);     /* number of symbol
                                          ** table entries */
 
         offset += nsyms * MACHO_NLIST_SIZE;
-        fwriteint32_t(offset, machofp);    /* string table offset */
-        fwriteint32_t(strslen, machofp);   /* string table size */
+        fwriteint32_t(offset, ofile);    /* string table offset */
+        fwriteint32_t(strslen, ofile);   /* string table size */
     }
 
     /* emit section data */
@@ -1254,7 +1229,7 @@ static void macho_write (void)
     /* we don't need to pad here since MACHO_NLIST_SIZE == 12 */
 
     /* emit string table */
-    saa_fpwrite(strs, machofp);
+    saa_fpwrite(strs, ofile);
 }
 /* We do quite a bit here, starting with finalizing all of the data
    for the object file, writing, and then freeing all of the data from
@@ -1280,9 +1255,6 @@ static void macho_cleanup(int debuginfo)
     macho_calculate_sizes();
     macho_write();
 
-    /* done - yay! */
-    fclose(machofp);
-
     /* free up everything */
     while (sects->next) {
         s = sects;
@@ -1343,12 +1315,12 @@ struct ofmt of_macho32 = {
     &null_debug_form,
     macho_stdmac,
     macho_init,
-    macho_setinfo,
+    null_setinfo,
     macho_output,
     macho_symdef,
     macho_section,
     macho_segbase,
-    macho_directive,
+    null_directive,
     macho_filename,
     macho_cleanup
 };
@@ -1361,12 +1333,12 @@ struct ofmt of_macho = {
     &null_debug_form,
     macho_stdmac,
     macho_init,
-    macho_setinfo,
+    null_setinfo,
     macho_output,
     macho_symdef,
     macho_section,
     macho_segbase,
-    macho_directive,
+    null_directive,
     macho_filename,
     macho_cleanup
 };
